#include "at32f403a_407.h"
#include "input.h"
#include "FreeRTOS.h"
#include "task.h"
#include <stdio.h>


in_t inputs[INPUT_NUMBER] = {
    {GPIOB, GPIO_PINS_11, 1, GPIO_PORT_SOURCE_GPIOB, GPIO_PINS_SOURCE11, 0, false, 0}, // 1   
    {GPIOB, GPIO_PINS_10, 1, GPIO_PORT_SOURCE_GPIOB, GPIO_PINS_SOURCE10, 0, false, 0}, // 2
    {GPIOE, GPIO_PINS_14, 1, GPIO_PORT_SOURCE_GPIOE, GPIO_PINS_SOURCE14, 0, false, 0}, // 3 перенесли {GPIOB, GPIO_PINS_1,  1, GPIO_PORT_SOURCE_GPIOB, GPIO_PINS_SOURCE1,  0},
    {GPIOE, GPIO_PINS_15, 1, GPIO_PORT_SOURCE_GPIOE, GPIO_PINS_SOURCE15, 0, false, 0}, // 4 перенесли {GPIOB, GPIO_PINS_0,  1, GPIO_PORT_SOURCE_GPIOB, GPIO_PINS_SOURCE0,  0},
    {GPIOA, GPIO_PINS_3,  1, GPIO_PORT_SOURCE_GPIOA, GPIO_PINS_SOURCE3,  0, false, 0}, // 5
    {GPIOA, GPIO_PINS_2,  1, GPIO_PORT_SOURCE_GPIOA, GPIO_PINS_SOURCE2,  0, false, 0}, // 6
    {GPIOA, GPIO_PINS_1,  1, GPIO_PORT_SOURCE_GPIOA, GPIO_PINS_SOURCE1,  0, false, 0}, // 7
    {GPIOA, GPIO_PINS_0,  1, GPIO_PORT_SOURCE_GPIOA, GPIO_PINS_SOURCE0,  0, false, 0}  // 8
};


//
void in_init(in_t *input)
{
    gpio_init_type gpio_init_struct;
    exint_init_type exint_init_struct;

    // Вход
    if (input->mode == 0)
    {
        gpio_default_para_init(&gpio_init_struct);
        
        gpio_init_struct.gpio_pull           = GPIO_PULL_NONE;  
        gpio_init_struct.gpio_mode           = GPIO_MODE_INPUT;  
        gpio_init_struct.gpio_drive_strength = GPIO_DRIVE_STRENGTH_STRONGER;
        gpio_init_struct.gpio_pins           = input->pin;
        gpio_init(input->port, &gpio_init_struct); 
    }
    // Счетный вход
    else if (input->mode == 1)
    {
        
    }

}

//
void in_exint_init(void)
{
    exint_init_type exint_init_struct;
    
    gpio_exint_line_config(GPIO_PORT_SOURCE_GPIOB, GPIO_PINS_SOURCE11); // 11
    gpio_exint_line_config(GPIO_PORT_SOURCE_GPIOB, GPIO_PINS_SOURCE10); // 10
    gpio_exint_line_config(GPIO_PORT_SOURCE_GPIOE, GPIO_PINS_SOURCE14); // 14
    gpio_exint_line_config(GPIO_PORT_SOURCE_GPIOE, GPIO_PINS_SOURCE15); // 15
    gpio_exint_line_config(GPIO_PORT_SOURCE_GPIOA, GPIO_PINS_SOURCE3);  // 3
    gpio_exint_line_config(GPIO_PORT_SOURCE_GPIOA, GPIO_PINS_SOURCE2);  // 2
    gpio_exint_line_config(GPIO_PORT_SOURCE_GPIOA, GPIO_PINS_SOURCE1);  // 1
    gpio_exint_line_config(GPIO_PORT_SOURCE_GPIOA, GPIO_PINS_SOURCE0);  // 0
    
    exint_default_para_init(&exint_init_struct);
    
    exint_init_struct.line_enable = TRUE;
    exint_init_struct.line_mode = EXINT_LINE_INTERRUPUT;
    exint_init_struct.line_select = EXINT_LINE_0 | EXINT_LINE_1 | EXINT_LINE_2 |
                                    EXINT_LINE_3 | EXINT_LINE_10 | 
                                    EXINT_LINE_11| EXINT_LINE_14 | EXINT_LINE_15; 
    
    exint_init_struct.line_polarity = EXINT_TRIGGER_RISING_EDGE;
    exint_init(&exint_init_struct);
    
    nvic_irq_enable(EXINT0_IRQn, 5, 0);
    nvic_irq_enable(EXINT1_IRQn, 5, 0);
    nvic_irq_enable(EXINT2_IRQn, 5, 0);
    nvic_irq_enable(EXINT3_IRQn, 5, 0);
    nvic_irq_enable(EXINT15_10_IRQn, 5, 0);
}

//
void io_test(void)
{
#if 0  
    printf("IN_1: %u IN_2: %u IN_3: %u IN_4: %u IN_5: %u IN_6: %u IN_7: %u IN_8: %u\r\n", 
           gpio_input_data_bit_read(inputs[0].port, inputs[0].pin),
           gpio_input_data_bit_read(inputs[1].port, inputs[1].pin),
           gpio_input_data_bit_read(inputs[2].port, inputs[2].pin),
           gpio_input_data_bit_read(inputs[3].port, inputs[3].pin),
           gpio_input_data_bit_read(inputs[4].port, inputs[4].pin),
           gpio_input_data_bit_read(inputs[5].port, inputs[5].pin),
           gpio_input_data_bit_read(inputs[6].port, inputs[6].pin), 
           gpio_input_data_bit_read(inputs[7].port, inputs[7].pin));
#endif    
    
    printf("IN_1: %u IN_2: %u IN_3: %u IN_4: %u IN_5: %u IN_6: %u IN_7: %u IN_8: %u\r\n", 
           input_cnt[0], input_cnt[1], input_cnt[2], input_cnt[3], 
           input_cnt[4], input_cnt[5], input_cnt[6], input_cnt[7]);
    
}

//
void input_task(void *params)
{
    for (;;)
    {
        for (int i = 0; i < INPUT_NUMBER; i++)
        {
            // Режим обычного входа
            if (inputs[i].mode == 0)
            {
                input_state[i] = (uint16_t)gpio_input_data_bit_read(inputs[i].port, inputs[i].pin);
            }
        }
            
        vTaskDelay(100);
    }
}

//
inline void debounce(void)
{
    for (int i = 0; i < INPUT_NUMBER; i++)
    {
        if (inputs[i].p_flag) {
            if (++inputs[i].cnt == DEBOUNCE_CNT) {
                inputs[i].cnt = 0;
                inputs[i].p_flag = false;
                input_cnt[i]++;
            }
        }
    }
}


void EXINT0_IRQHandler(void)
{
    if (exint_flag_get(EXINT_LINE_0) != RESET) // IN_8 GPIOA_0
    {
        exint_flag_clear(EXINT_LINE_0);
        inputs[7].p_flag = true;
    }
}

void EXINT1_IRQHandler(void)
{
    if(exint_flag_get(EXINT_LINE_1) != RESET) // IN_7 GPIOA_1
    {
        exint_flag_clear(EXINT_LINE_1);
        inputs[6].p_flag = true;
    }
}

void EXINT2_IRQHandler(void)
{
    if(exint_flag_get(EXINT_LINE_2) != RESET) // IN_6 GPIOA_2
    {
        exint_flag_clear(EXINT_LINE_2);
        inputs[5].p_flag = true;
    }
}

void EXINT3_IRQHandler(void)
{
    if(exint_flag_get(EXINT_LINE_3) != RESET) // IN_5 GPIOA_3
    {
        exint_flag_clear(EXINT_LINE_3);
        inputs[4].p_flag = true;
    }
}

void EXINT15_10_IRQHandler(void)
{
    if (exint_flag_get(EXINT_LINE_10) != RESET)  // IN_2 GPIOB_10
    {
        exint_flag_clear(EXINT_LINE_10);
        inputs[1].p_flag = true;
    }
    else if (exint_flag_get(EXINT_LINE_11) != RESET)  // IN_1 GPIOB_11
    {
        exint_flag_clear(EXINT_LINE_11);
        inputs[0].p_flag = true;
    }
    else if (exint_flag_get(EXINT_LINE_14) != RESET)  // IN_3 GPIOE_14
    {
        exint_flag_clear(EXINT_LINE_14);
        inputs[2].p_flag = true;
    }
    else if (exint_flag_get(EXINT_LINE_15) != RESET)  // IN_4 GPIOE_15
    {
        exint_flag_clear(EXINT_LINE_15);
        inputs[3].p_flag = true;
    }
    
}
